#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## Default initialization for flow tests
#
# Adrien Crovato

from fwk.wutils import parseargs
import flow as flo

## Initialize Picard solver
def picard(pbl):
    from tbox import Gmres
    args = parseargs()
    solver = flo.Picard(pbl, Gmres())
    solver.nthreads = args.k
    solver.verbose = args.verb
    solver.relTol = 1e-6
    solver.absTol = 1e-8
    solver.maxIt = 25
    solver.relax = 0.7
    return solver

## Initialize Newton solver
def newton(pbl):
    from tbox.solvers import LinearSolver
    args = parseargs()
    solver = flo.Newton(pbl, LinearSolver().pardiso())
    solver.nthreads = args.k
    solver.verbose = args.verb
    solver.relTol = 1e-6
    solver.absTol = 1e-8
    solver.maxIt = 25
    solver.lsTol = 1e-6
    solver.maxLsIt = 10
    solver.avThrsh = 1e-2
    return solver

## Initialize mesh deformer
def initMeshDeformer(_mshDef):
    args = parseargs()
    _mshDef.nthreads = args.k

## Initialize viewer
def initViewer(problem):
    args = parseargs()
    if not args.nogui:
        from tbox.viewer import GUI
        GUI().open( problem.msh.name )
