/* 2D5 Cylinder */

// Parameters
// domain
DefineConstant[ dmt = { 1.0, Name "Cylinder diameter" }  ];
DefineConstant[ lgt = { 3.0, Name "Channel length" }  ];
DefineConstant[ wdt = { 2.0, Name "Channel width" }  ];
DefineConstant[ hgt = { 3.0, Name "Channel height" }  ];
DefineConstant[ msN = { 0.1, Min 0.1, Max 1, Step 0.1, Name "Nearfield mesh size" }  ];
DefineConstant[ msF = { 1.0, Min 0.1, Max 1, Step 0.1, Name "Farfield mesh size" }  ];

// mesh algo
msh3D = 2;
msh2D = 5;

// Geometry
// cylinder
Point(0) = {0, 0, 0};
Point(1) = {dmt/2, 0, 0, msN};
Point(2) = {0, 0, dmt/2, msN};
Point(3) = {-dmt/2, 0, 0, msN};
Point(4) = {0, 0, -dmt/2, msN};
Point(10) = {0, wdt, 0};
Point(11) = {dmt/2, wdt, 0, msN};
Point(12) = {0, wdt, dmt/2, msN};
Point(13) = {-dmt/2, wdt, 0, msN};
Point(14) = {0, wdt, -dmt/2, msN};

// box
Point(31) = {lgt/2, 0, hgt/2, msF};
Point(32) = {-lgt/2, 0, hgt/2, msF};
Point(33) = {-lgt/2, 0, -hgt/2, msF};
Point(34) = {lgt/2, 0, -hgt/2, msF};
Point(35) = {lgt/2, wdt, hgt/2, msF};
Point(36) = {-lgt/2, wdt, hgt/2, msF};
Point(37) = {-lgt/2, wdt, -hgt/2, msF};
Point(38) = {lgt/2, wdt, -hgt/2, msF};

// midplane
Point(41) = {lgt/2, 0, 0, msF};
Point(42) = {lgt/2, wdt, 0, msF};
Point(43) = {-lgt/2, wdt, 0, msF};
Point(44) = {-lgt/2, 0, 0, msF};

// wing
Circle(1) = {1, 0, 2};
Circle(2) = {2, 0, 3};
Circle(3) = {3, 0, 4};
Circle(4) = {4, 0, 1};
Circle(5) = {11, 10, 12};
Circle(6) = {12, 10, 13};
Circle(7) = {13, 10, 14};
Circle(8) = {14, 10, 11};
Line(9) = {1, 11};
Line(10) = {2, 12};
Line(11) = {3, 13};
Line(12) = {4, 14};

// box
Line(21) = {41, 31};
Line(22) = {31, 32};
Line(23) = {32, 44};
Line(24) = {44, 33};
Line(25) = {33, 34};
Line(26) = {34, 41};
Line(27) = {42, 35};
Line(28) = {35, 36};
Line(29) = {36, 43};
Line(30) = {43, 37};
Line(31) = {37, 38};
Line(32) = {38, 42};
Line(33) = {31, 35};
Line(34) = {32, 36};
Line(35) = {33, 37};
Line(36) = {34, 38};

// midplane
Line(41) = {1, 41};
Line(42) = {11, 42};
Line(43) = {13, 43};
Line(44) = {3, 44};
Line(45) = {41, 42};
Line(46) = {42, 11};
Line(47) = {13, 43};
Line(48) = {43, 44};

// wing
Line Loop(1) = {1, 10, -5, -9};
Surface(1) = {-1};
Line Loop(2) = {2, 11, -6, -10};
Surface(2) = {-2};
Line Loop(3) = {3, 12, -7, -11};
Surface(3) = {-3};
Line Loop(4) = {4, 9, -8, -12};
Surface(4) = {-4};

// symmetry
Line Loop(5) = {41, 21, 22, 23, -44, -2, -1};
Plane Surface(5) = {-5};
Line Loop(12) = {42, 27, 28, 29, -43, -6, -5};
Plane Surface(12) = {12};
Line Loop(6) = {44, 24, 25, 26, -41, -4, -3};
Plane Surface(6) = {-6};
Line Loop(13) = {43, 30, 31, 32, -42, -8, -7};
Plane Surface(13) = {13};

// upstream
Line Loop(7) = {48, -23, 34, 29};
Plane Surface(7) = {-7};
Line Loop(8) = {48, 24, 35, -30};
Plane Surface(8) = {8};

// downstream
Line Loop(9) = {45, 27, -33, -21};
Plane Surface(9) = {-9};
Line Loop(10) = {45, -32, -36, 26};
Plane Surface(10) = {10};

// farfield
Line Loop(11) = {22, 34, -28, -33};
Plane Surface(11) = {11};
Line Loop(14) = {25, 36, -31, -35};
Plane Surface(14) = {14};

// wake
Line Loop(15) = {41, 45, -42, -9};
Plane Surface(15) = {15};

// internal
Line Loop(16) = {11, 43, 48, -44};
Plane Surface(16) = {16};

// field
Surface Loop(1) = {7, 5, 9, 12, 11, 2, 1, 16, 15};
Volume(1) = {1};
Surface Loop(2) = {6, 8, 14, 10, 13, 4, 3, 16, 15};
Volume(2) = {2};

// Mesh
// 2D
MeshAlgorithm Surface{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16} = msh2D;
// 3D
Mesh.Algorithm3D = msh3D;
Mesh.OptimizeNetgen = 1;
Mesh.Smoothing = 10;
Mesh.SmoothNormals = 1;

// Physical
Physical Line("te") = {9};
Physical Surface("cylinder") = {1, 2};
Physical Surface("cylinder_") = {3, 4};
Physical Surface("symmetry") = {5, 12};
Physical Surface("symmetry_") = {6, 13};
Physical Surface("upstream") = {7, 8};
Physical Surface("downstream") = {9};
Physical Surface("downstream_") = {10};
Physical Surface("side") = {11, 14};
Physical Surface("wake") = {15};
Physical Volume("field") = {1};
Physical Volume("field_") = {2};

