/* 3D channel with cylinder */

// Parameters
DefineConstant[ dmt = { 0.5, Name "Cylinder diameter" }  ];
DefineConstant[ spn = { 1.5, Name "Cylinder span" }  ];
DefineConstant[ lgt = { 4.0, Name "Channel length" }  ];
DefineConstant[ wdt = { 2.5, Name "Channel width" }  ];
DefineConstant[ hgt = { 3.0, Name "Channel height" }  ];
DefineConstant[ mshN = { 0.1, Min 0.1, Max 1, Step 0.1, Name "Nearfield mesh size" }  ];
DefineConstant[ mshF = { 0.5, Min 0.1, Max 1, Step 0.1, Name "Farfield mesh size" }  ];


// Geometry

// center
Point(0) = {0., 0., 0.};
Point(5) = {0., spn, 0.};
// cylinder
Point(1) = {dmt/2, 0., 0., mshN};
Point(2) = {0., 0., dmt/2, mshN};
Point(3) = {-dmt/2, 0., 0., mshN};
Point(4) = {0., 0., -dmt/2, mshN};
Point(6) = {dmt/2, spn, 0., mshN};
Point(7) = {0., spn, dmt/2, mshN};
Point(8) = {-dmt/2, spn, 0., mshN};
Point(9) = {0., spn, -dmt/2, mshN};
// symmetry
Point(11) = {lgt/2, 0., hgt/2, mshF};
Point(12) = {-lgt/2, 0., hgt/2, mshF};
Point(13) = {-lgt/2, 0., -hgt/2, mshF};
Point(14) = {lgt/2, 0., -hgt/2, mshF};
// farfield
Point(16) = {lgt/2, wdt, hgt/2, mshF};
Point(17) = {-lgt/2, wdt, hgt/2, mshF};
Point(18) = {-lgt/2, wdt, -hgt/2, mshF};
Point(19) = {lgt/2, wdt, -hgt/2, mshF};

// cylinder
Circle(1) = {1, 0, 2};
Circle(2) = {2, 0, 3};
Circle(3) = {3, 0, 4};
Circle(4) = {4, 0, 1};
Line(6) = {1, 6};
Line(7) = {2, 7};
Line(8) = {3, 8};
Line(9) = {4, 9};
Circle(11) = {6, 5, 7};
Circle(12) = {7, 5, 8};
Circle(13) = {8, 5, 9};
Circle(14) = {9, 5, 6};
// box
Line(21) = {11, 12};
Line(22) = {12, 13};
Line(23) = {13, 14};
Line(24) = {14, 11};
Line(26) = {11, 16};
Line(27) = {12, 17};
Line(28) = {13, 18};
Line(29) = {14, 19};
Line(31) = {16, 17};
Line(32) = {17, 18};
Line(33) = {18, 19};
Line(34) = {19, 16};

// cylinder
Line Loop(1) = {1, 7, -11, -6};
Line Loop(2) = {2, 8, -12, -7};
Line Loop(3) = {3, 9, -13, -8};
Line Loop(4) = {4, 6, -14, -9};
Surface(1) = {-1};
Surface(2) = {-2};
Surface(3) = {-3};
Surface(4) = {-4};
Line Loop(5) = {11, 12, 13, 14};
Surface(5) = {-5};
// symmetry
Line Loop(11) = {1, 2, 3, 4};
Line Loop(12) = {21, 22, 23, 24};
Surface(11) = {-12, 11};

// inlet
Line Loop(13) = {-22, 27, 32, -28};
Surface(12) = {-13};
// outlet
Line Loop(14) = {24, 26, -34, -29};
Surface(13) = {14};
// top
Line Loop(15) = {-21, 26, 31, -27};
Surface(14) = {-15};
// bottom
Line Loop(16) = {23, 29, -33, -28};
Surface(15) = {16};
// farfield
Line Loop(17) = {31, 32, 33, 34};
Surface(16) = {17};

// Field
Surface Loop(1) = {1, 2, 3, 4, 5, 11, 12, 13, 14, 15, 16};
Volume(1) = {1};

// Physical groups
internalField = 1000;
inlet = 1001;
outlet = 1002;
side = 1003;
symmetry = 1004;
farfield = 1005;
cylinder = 1006;

Physical Volume("internalField", internalField) = {1};
Physical Surface("upstream", inlet) = {12};
Physical Surface("downstream", outlet) = {13};
Physical Surface("side", side) = {14, 15};
Physical Surface("symmetry", symmetry) = {11};
Physical Surface("farfield", farfield) = {16};
Physical Surface("cylinder", cylinder) = {1, 2, 3, 4, 5};