/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wF0Ps.h"
#include <sstream>
using namespace tbox;
using namespace flow;

/**
 * @brief Return constant function
 * @authors Adrien Crovato
 */
double F0PsC::eval(Eigen::Vector3d const &pos) const
{
    return v;
}
void F0PsC::write(std::ostream &out) const
{
    out << "F0PsC";
}

F0PsPhiInf::F0PsPhiInf(int _nDim, double _alpha) : F0Ps(), nDim(_nDim), alpha(_alpha)
{
    // Sanity check
    if (nDim != 2 && nDim != 3)
    {
        std::stringstream err;
        err << "F0PsPhiInf::nDim should be equal to 2 or 3 but " << nDim << " was given!\n";
        throw std::runtime_error(err.str());
    }
}
/**
 * @brief Update the angle of attack
 * @authors Adrien Crovato
 */
void F0PsPhiInf::update(double _alpha)
{
    alpha = _alpha;
}
/**
 * @brief Evaluate the freestream potential
 * @authors Adrien Crovato
 */
double F0PsPhiInf::eval(Eigen::Vector3d const &pos) const
{
    return cos(alpha) * pos(0) + sin(alpha) * pos(nDim - 1);
}
void F0PsPhiInf::write(std::ostream &out) const
{
    out << "F0PsPhiInf";
}
