#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# @brief Viscous-inviscid coupler
# @authors Adrien Crovato, Amaury Bilocq

from __future__ import print_function
from builtins import range
from builtins import object
import numpy as np
from fwk.coloring import ccolors

class Coupler(object):
    def __init__(self, _isolver, _vsolver, _writer):
        '''...
        '''
        self.isolver =_isolver # inviscid solver
        self.vsolver = _vsolver # viscous solver
        self.writer = _writer

    def run(self):
        ''' Perform coupling
        '''
        # initialize loop
        it = 0
        converged = False # temp
        while True:
            print(ccolors.ANSI_BLUE + 'Iteration: ', it, ccolors.ANSI_RESET)
            # run inviscid solver
            self.isolver.run()
            # get velocity at edge of BL from inviscid solver and update viscous solver
            for i in range(0, len(self.vsolver.boundary.nodes)):
                self.vsolver.v[i,0] = self.isolver.U[self.vsolver.boundary.nodes[i].row][0]
                self.vsolver.v[i,1] = self.isolver.U[self.vsolver.boundary.nodes[i].row][1]
                self.vsolver.v[i,2] = self.isolver.U[self.vsolver.boundary.nodes[i].row][2]
            # run viscous solver
            self.vsolver.run()
            # get blowing velocity from viscous solver and update inviscid solver
            for i in range(0, self.vsolver.nE):
                self.vsolver.boundary.setU(i, self.vsolver.u[i])
            # convergence test
            if converged:
                break
            else:
                converged = True # perform 2 iterations
                it += 1
            
        # save results
        self.isolver.save(0, self.writer)
        print('\n')
