/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wTimes.h"

#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h> // needed by Linux 32-bit only!
#include <sys/times.h>
#endif

using namespace fwk;

Times Times::read()
{
#ifdef WIN32
    HANDLE hProcess = GetCurrentProcess();
    FILETIME ftCreation, ftExit, ftKernel, ftUser;
    GetProcessTimes(hProcess, &ftCreation, &ftExit, &ftKernel, &ftUser);
    user = Int64Time(*(reinterpret_cast<__int64 *>(&ftUser)));
    kernel = Int64Time(*(reinterpret_cast<__int64 *>(&ftKernel)));
    real = ClockTime(clock());
#else
    struct tms tm;
    real = UnixClockTime(times(&tm));
    user = UnixClockTime(tm.tms_utime);
    kernel = UnixClockTime(tm.tms_stime);
#endif
    return *this;
}

namespace fwk
{
FWK_API std::ostream &
operator<<(std::ostream &out, Times const &obj)
{
    out << "usr:" << obj.getUser();
    out << " kern:" << obj.getKernel();
    out << " real:" << obj.getReal();
    return out;
}
} // namespace fwk
