#!/usr/bin/env python
# -*- coding: utf-8 -*-
# simple RVE with 2 materials for testing periodic BCs

from __future__ import print_function
import heat as h
import tbox
import tbox.gmsh as gmsh

def main():

    msh = gmsh.MeshLoader("jalil1_micro.geo", __file__).execute()

    pbl = h.Problem(msh)

    solver = h.Solver(pbl)
    solver.nthreads = 4

    h.Medium(pbl, "Matrix",    tbox.Fct2C(10.,10.,0), 1.0)
    h.Medium(pbl, "Inclusion", tbox.Fct2C(140.,140.,0), 1.0)

    TM   = 0.0
    dTxM = 1.0
    dTyM = 1.0
    p = h.Periodic(pbl, TM, dTxM, dTyM)

    solver.start()
    print("flux=", p.qM)
    
    from heat.viewer import GUI
    GUI().open('jalil1_micro')

if __name__ == "__main__":
    main()
