#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import katropton.CMAME.model as model

import katropton as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katropton.utilities as u


def evaluate_prec(msh, comm, file_dir, Ngs, omegags, ensemble_size=1):

    work_dir = os.getcwd()
    args = parseargs()

    p = model.defaut_params

    N = ensemble_size
    num_random_variables = 20
    randomVariable = np.zeros((N, num_random_variables))
    if ensemble_size is not 1:
        nuw = 128
        p['Correlation length'] = 5.
        p['Dispersion level'] = 0.1

        p['Wavenumber cutoff'] = 8.
        p['Wavenumber discretization points'] = nuw
        p['Number Z of previously drawn samples'] = 0
        p['Number Phi of previously drawn samples'] = 0
        p['Is exp'] = False

    pbl = m.Problem(msh, comm)

    # Units are in mm
    E = 70000 * np.ones(ensemble_size)  # 70e9 Pa = 70000 MPa
    nu = 0.35 * np.ones(ensemble_size)
    k = 0.
    d = 0.
    m.Medium(pbl, "body", "test", E, nu, k, d)

    m.Dirichlet(pbl, "clamped xyz", "Clamped", 1, 0.,
                1, 0., 1, 0., 0, 0., ensemble_size)
    m.Dirichlet(pbl, "clamped xy", "Clamped", 1, 0.,
                1, 0., 0, 0., 0, 0., ensemble_size)
    m.Dirichlet(pbl, "clamped y", "Clamped", 0, 0.,
                1, 0., 0, 0., 0, 0., ensemble_size)

    f = 700 * np.ones(ensemble_size)  # 700e6 Pa = 700 MPa
    zero = np.zeros(ensemble_size)
    one = np.ones(ensemble_size)
    m.Neumann(pbl, 'load 1', 'load', 0, zero, 1,
              f, 0, zero, 0, one, ensemble_size)
    m.Neumann(pbl, 'load 2', 'load', 0, zero, 1, -
              f, 0, zero, 0, one, ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Ensemble Convergence Tolerance'] = 1e-6
    solverList['Maximum Iterations'] = 1000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    mueluParams = Teuchos.ParameterList()
    mueluParams['number of equations'] = 3
    mueluParams['max levels'] = 1
    mueluParams['coarse: type'] = "RELAXATION"

    mueluParamsSub2 = Teuchos.ParameterList()
    mueluParamsSub2['relaxation: type'] = "MT Gauss-Seidel"
    mueluParamsSub2['relaxation: sweeps'] = Ngs
    mueluParamsSub2['relaxation: damping factor'] = omegags

    mueluParams['verbosity'] = "high"

    mueluParams['coarse: params'] = mueluParamsSub2

    solverList['convert MueLu xml file'] = True
    solverList['mueluParams'] = mueluParams

    solverList['Print Teuchos timers'] = True

    solverList['gamma'] = p['gamma']
    solverList['Use preconditioner'] = True

    solverList['type'] = 'BlockGmres'
    solverList['Maximum active set iteration'] = 1

    solverList['Write matrix and vectors'] = p['Write matrix and vectors']
    solverList['Write txt files'] = p['Write txt files']

    if p["Is exp"]:
        num_random_variables = randomVariable.shape[1]
    else:
        num_random_variables = 0

    for i in range(0, num_random_variables):
        rand = randomVariable[:, i]
        pbl.add(m.RandomVariable(rand))

    randomParams = Teuchos.ParameterList()

    randomParams['Is random'] = True
    randomParams['Is exp'] = p['Is exp']
    if p["Is exp"]:
        randomParams['Begin X'] = 0.
        randomParams['Begin Y'] = 0.
        randomParams['Begin Z'] = 0.
        randomParams['End X'] = 10.
        randomParams['End Y'] = 10.
        randomParams['End Z'] = 1.
        randomParams['Number random variables'] = num_random_variables
        randomParams['Correlation length X'] = p['Correlation length X']
        randomParams['Correlation length Y'] = p['Correlation length Y']
        randomParams['Correlation length Z'] = p['Correlation length Z']
        randomParams['Number of KL Terms X'] = p['Number of KL Terms X']
        randomParams['Number of KL Terms Y'] = p['Number of KL Terms Y']
        randomParams['Number of KL Terms Z'] = p['Number of KL Terms Z']
        randomParams['Maximum Nonlinear Solver Iterations'] = 5000
        randomParams['Bound Perturbation Size'] = 1E-10
    else:
        randomParams['Correlation length'] = p['Correlation length']
        randomParams['Wavenumber cutoff'] = p['Wavenumber cutoff']
        randomParams['Wavenumber discretization points'] = p['Wavenumber discretization points']
        randomParams['Number Z of previously drawn samples'] = p['Number Z of previously drawn samples']
        randomParams['Number Phi of previously drawn samples'] = p['Number Phi of previously drawn samples']

    randomParams['Mean'] = old_div(E[0], (2.*(1.+nu[0])))
    randomParams['Dispersion level'] = p['Dispersion level']

    solverList['randomParams'] = randomParams

    slv = m.IterativeSolver(pbl, args.k, solverList, 3, ensemble_size)
    slv.start()
    f = open('timers.txt', 'w')
    print(slv.getTimers(), file=f)
    f.closed


def evaluate_all_prec(msh, comm, file_dir, prec_params):

    N = prec_params.shape[0]

    for i in range(500, 1000):
        directory = str(i)
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)

        current_dir = os.getcwd()

        Ngs = int(prec_params[i, 0])
        omegags = prec_params[i, 1]

        directory = 'mean_case'
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)
        evaluate_prec(msh, comm, file_dir, Ngs, omegags)
        os.chdir('..')

        for ensemble_size in [8]:
            directory = 'ensemble_reduction_' + str(ensemble_size)
            if not os.path.exists(directory):
                os.makedirs(directory)
            os.chdir(directory)
            evaluate_prec(msh,
                          comm,
                          file_dir,
                          Ngs, omegags,
                          ensemble_size=ensemble_size)
            os.chdir('..')

        os.chdir('..')


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")

    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = gmsh.MeshLoader('plate.geo', file_dir).execute()

    GS_sweeps_00_min = 1
    GS_sweeps_00_max = 41
    delta_GS_sweeps_00 = 1

    N_GS_sweeps_00 = int(
        np.ceil(
            old_div((GS_sweeps_00_max - GS_sweeps_00_min * 1.), delta_GS_sweeps_00)))

    # Damping has to be between 0 and 2
    GS_damping_00_min = 0.001
    GS_damping_00_max = 1.999
    N_GS_damping_00 = 50

    sweeps_00 = np.arange(GS_sweeps_00_min, GS_sweeps_00_max,
                          delta_GS_sweeps_00)
    damping_00 = np.linspace(GS_damping_00_min, GS_damping_00_max,
                             N_GS_damping_00)

    N = N_GS_sweeps_00 * N_GS_damping_00

    prec_params = np.zeros((N, 2))

    i = 0

    for i_0 in range(0, N_GS_sweeps_00):
        for i_1 in range(0, N_GS_damping_00):
            prec_params[i, :] = [
                sweeps_00[i_0], damping_00[i_1]
            ]
            i = i + 1

    np.savetxt('prec_params.txt', prec_params)

    evaluate_all_prec(msh, comm, file_dir, prec_params)


if __name__ == "__main__":
    main()
