#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
from katropton.eigenvalues import *
import os
import fwk


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    N = 32

    input_file_dir = file_dir + '/../../workspace/katropton_CMAME_tests_plate_MC_NO_EP/1/'

    sample_per_rank = np.ceil(old_div(N, siz))
    i_min = rank*sample_per_rank
    i_max = (rank+1)*sample_per_rank

    if i_min > N:
        i_min = N
    if i_max > N:
        i_max = N

    i_min = int(i_min)
    i_max = int(i_max)

    for i in range(i_min, i_max):
        timers = fwk.Timers()

        filename_Q = input_file_dir + str(i) + '/ApCTC_mm.txt'
        filename_GT = input_file_dir + str(i) + '/B_mm.txt'
        filename_f = input_file_dir + str(i) + '/b_mm_after_bc.txt'

        timers['Read files'].start()
        Q = read_mm(filename_Q)
        Q = get_symmetrical_part(Q)
        GT = read_mm(filename_GT)
        G = GT.transpose()
        f = read_mm(filename_f, is_sparse=False)
        timers['Read files'].stop()

        timers['Compute R'].start()
        R = approx_Schur_complement(Q, G, GT)
        timers['Compute R'].stop()

        timers['Compute invR'].start()
        invR = compute_inv(R)
        timers['Compute invR'].stop()

        print(timers)

        timers['Compute the inverse of Q'].start()
        invQG = compute_inv_on_image(Q, G)
        timers['Compute the inverse of Q'].stop()

        print(timers)

        timers['Compute the preconditioned matrix'].start()
        AinvP = SIMPLE_prec(Q, invQG, G, GT, R, invR, True)
        timers['Compute the preconditioned matrix'].stop()

        print(timers)

        timers['Compute the basis'].start()

        n = Q.shape[0]+R.shape[0]
        b = np.zeros((n,))
        b[0:len(f)] = f

        N = 1
        V = compute_V(AinvP, b, m=0, N=N)
        W = compute_W(AinvP, b, V, N=N)

        timers['Compute the basis'].stop()

        print(timers)

        timers['Compute the eigenvalues'].start()
        deltas, omegas = compute_deltas_omegas(V, W, b)
        timers['Compute the eigenvalues'].stop()

        print(timers)

        timers['Write the eigenvalues'].start()
        write_eigenvalues(deltas, work_dir +
                          '/deltas_direct_direct_' + str(i) + '.txt')
        np.savetxt(work_dir+'/omegas_direct_direct_' + str(i) + '.txt', omegas)
        timers['Write the eigenvalues'].stop()

        f = open('timers_' + str(i) + '.txt', 'w')
        print(timers, file=f)
        f.closed


if __name__ == "__main__":
    main()
