#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
from katropton.eigenvalues import *
import os
import fwk
"""@package docstring
This file is used to loop over cases and samples and plot their mechanical properties and solution fields.
"""

import vtk
import tboxVtk
import tboxVtk.reader as vtkR
import tboxVtk.cutter as vtkC

from katropton.CMAME.model import Compute_QoIs


def plot_2D_mesh(pts, quad, ax):
    """
    This function plots a 2D quad mesh on a given matplotlib axis
    """
    import matplotlib.pyplot as plt
    for i in range(0, quad.shape[0]):
        x = np.zeros((quad.shape[1] + 1, ))
        y = np.zeros((quad.shape[1] + 1, ))
        for j in range(0, quad.shape[1]):
            x[j] = pts[quad[i, j], 0]
            y[j] = pts[quad[i, j], 1]
        x[-1] = pts[quad[i, 0], 0]
        y[-1] = pts[quad[i, 0], 1]
        ax.plot(x, y, '--k', linewidth=0.5)


def cut_of_one_case(case_id,
                    ensemble_size,
                    n_samples,
                    iters,
                    file_MC_base_dir):
    """
    This function loops over the samples of a given test case and plot them
    """
    import matplotlib.pyplot as plt

    base_directory = 'case_' + str(case_id)
    if not os.path.exists(base_directory):
        os.makedirs(base_directory)
        os.makedirs(base_directory + '/sample')
        os.makedirs(base_directory + '/sample_ER_dif')
    base_directory_sample = base_directory + '/sample'
    base_directory_sample_ER_dif = base_directory + '/sample_ER_dif'
    for i in range(0, n_samples):

        input_MC_file_dir = file_MC_base_dir + \
            str(case_id) + '/1/' + str(i) + '/104/'
        mshName_sample_no_EP = input_MC_file_dir + 'plate_ast1_s0_0'
        pts, tri, vals = cut_plate(mshName_sample_no_EP, False)
        pts_2, quad = extract_quad_msh(mshName_sample_no_EP)

        x = pts[:, 0]
        y = pts[:, 1]

        plot_one_sampel(x, y, tri, vals, pts_2, quad, False)
        plt.savefig(base_directory_sample + '/' + str(i) + '.png', dpi=400)

        for i_it in range(0, len(iters)):

            i_ensemble = old_div(i, ensemble_size)
            i_sample = np.mod(i, ensemble_size)

            #input_MC_file_dir = file_MC_base_dir + str(case_id) + '/1/' + str(i) + '/' + str(iters[i_it]) + '/'

            input_MC_file_dir = file_MC_base_dir + str(case_id) + '/' + str(
                ensemble_size) + '/' + str(i_ensemble) + '/' + str(iters[i_it]) + '/'
            mshName_sample_EP = input_MC_file_dir + 'plate_ast1_s' + str(
                i_sample) + '_0'

            pts_EP, tri_EP, vals_EP = cut_plate(mshName_sample_EP,  False)

            plot_one_sampel(x, y, tri, vals_EP, pts_2, quad, False)
            plt.savefig(base_directory_sample + '/' + str(i) + '_' +
                        str(iters[i_it]) + '_e_' + str(ensemble_size) + '.png', dpi=400)

            plot_diff_abs(x, y, tri, vals, vals_EP, pts_2, quad, False)
            plt.savefig(base_directory_sample_ER_dif + '/' + str(i) + '_' +
                        str(iters[i_it]) + '_e_' + str(ensemble_size) + '.png', dpi=400)

            input_MC_file_dir = file_MC_base_dir + \
                str(case_id) + '/1/' + str(i) + '/' + str(iters[i_it]) + '/'
            mshName_sample_EP = input_MC_file_dir + 'plate_ast1_s0_0'

            pts_EP, tri_EP, vals_EP = cut_plate(mshName_sample_EP,  False)

            plot_one_sampel(x, y, tri, vals_EP, pts_2, quad, False)
            plt.savefig(base_directory_sample + '/' + str(i) +
                        '_' + str(iters[i_it]) + '_e_1.png', dpi=400)

            plot_diff_abs(x, y, tri, vals, vals_EP, pts_2, quad, False)
            plt.savefig(base_directory_sample_ER_dif + '/' +
                        str(i) + '_' + str(iters[i_it]) + '_e_1.png', dpi=400)

            plt.close('all')


def extract_quad_msh(mshName):
    """
    This function extracts the 2D quad mesh out of a hexahedron mesh
    """
    reader = vtkR.Reader()
    reader.open(mshName)
    rdata = reader.reader.GetOutput()

    _pts = rdata.GetPoints()
    pts = np.zeros((_pts.GetNumberOfPoints(), 2))
    pts_indices = np.zeros((_pts.GetNumberOfPoints(), ), dtype=int)
    for i in range(0, pts.shape[0]):
        if _pts.GetPoint(i)[2] <= 0.01:
            for j in range(0, 2):
                pts[i][j] = _pts.GetPoint(i)[j]
            pts_indices[i] = 1
    n_elems = rdata.GetNumberOfCells()
    quad = []
    for i in range(0, n_elems):
        elem = rdata.GetCell(i)
        for j in range(0, 6):
            face = elem.GetFace(j)
            points_id = np.zeros((4, ), dtype=int)
            good_face = True
            for k in range(0, 4):
                points_id[k] = face.GetPointId(k)
                if pts_indices[points_id[k]] != 1:
                    good_face = False
                    break
            if good_face:
                if len(quad) == 0:
                    quad = points_id.reshape((1, 4))
                else:
                    quad = np.append(quad, points_id.reshape((1, 4)), axis=0)
    return pts, quad


def cut_plate(mshName, at_point=False):
    """
    This function cuts the vtk files in the middle and extracts both point
    and element data.
    """
    reader = vtkR.Reader()
    reader.open(mshName)
    cutter = vtkC.Cutter(reader.reader.GetOutputPort())
    c_output = cutter.cut(1, [0., 0., 0.5], [0., 0., 1.], tag_name='volume_id')
    if at_point:
        pts, tri, vals = cutter.extract(c_output,
                                        3,
                                        ['u', 'x', 'y', 'z', 'G', 'von Mises'],
                                        atPoint=True)
    else:
        pts, tri, vals = cutter.extract(c_output,
                                        3, ['u', 'x', 'y', 'z'],
                                        atPoint=True)
        pts_2, tri_2, vals_2 = cutter.extract(c_output,
                                              3, ['G', 'von Mises'],
                                              atPoint=False)
        vals.update(vals_2)

    return pts, tri, vals


def plot_2D_cut_values_at_node(x, y, tri, value, n_levels=16):
    """
    This function plots data at node.
    """
    import matplotlib.pyplot as plt
    value = value.reshape((len(x), ))
    if max(value) - min(value) > 0:
        levels = np.linspace(min(value), max(value), n_levels)
        plt.set_cmap('coolwarm')
        plt.tricontour(x,
                       y,
                       tri,
                       value,
                       levels,
                       linestyles='-',
                       colors='black',
                       linewidths=0.5)
        axtri = plt.tricontourf(x, y, tri, value, levels)
    else:
        axtri = plt.tricontourf(x, y, tri, value)
    plt.gca().set_aspect('equal')
    return axtri


def plot_2D_cut_values_at_elem(x,
                               y,
                               tri_1,
                               value,
                               v_min=0,
                               v_max=0,
                               n_levels=16,
                               shading=False):
    """
    This function plots data at element.
    """
    import matplotlib.pyplot as plt

    value = value.reshape((tri_1.shape[0], ))
    if v_min == v_max:
        v_min = min(value)
        v_max = max(value)

    levels = np.linspace(v_min, v_max, n_levels)
    plt.set_cmap('coolwarm')
    if shading:
        axtri = plt.tripcolor(x, y, tri_1, value, shading='gouraud')
    else:
        axtri = plt.tripcolor(x, y, tri_1, value, levels)
    plt.gca().set_aspect('equal')
    return axtri


def plot_one_sampel(x, y, tri, vals_s, pts_2, quad, at_point=False):
    """
    This function plots data of a given sample.
    """
    import matplotlib.pyplot as plt

    fig = plt.figure(figsize=(15, 4))

    ax = plt.subplot(131)
    if not at_point:
        tmp = plot_2D_cut_values_at_elem(x, y, tri, vals_s['G'])
        plot_2D_mesh(pts_2, quad, ax)
    else:
        tmp = plot_2D_cut_values_at_node(x, y, tri, vals_s['G'])
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal')
    plt.title('G')

    plt.subplot(132)
    tmp = plot_2D_cut_values_at_node(x, y, tri, vals_s['y'])
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal', format='%.0e')
    plt.title('y')

    ax = plt.subplot(133)
    if not at_point:
        tmp = plot_2D_cut_values_at_elem(x, y, tri, vals_s['von Mises'])
        plot_2D_mesh(pts_2, quad, ax)
    else:
        tmp = plot_2D_cut_values_at_node(x, y, tri, vals_s['von Mises'])
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal')
    plt.title('von Mises')


def plot_diff_abs(x, y, tri, vals, vals_s, pts_2, quad, at_point=False):
    """
    This function plots the difference between data of a given sample and the mean sample.
    """
    import matplotlib.pyplot as plt

    fig = plt.figure(figsize=(15, 4))

    plt.subplot(131)
    tmp = plot_2D_cut_values_at_node(x, y, tri, vals_s['x'] - vals['x'])
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal', format='%.0e')
    plt.title('delta x')

    plt.subplot(132)
    tmp = plot_2D_cut_values_at_node(x, y, tri, vals_s['y'] - vals['y'])
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal', format='%.0e')
    plt.title('delta y')

    plt.subplot(133)
    tmp = plot_2D_cut_values_at_node(x, y, tri, vals_s['z'] - vals['z'])
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal', format='%.0e')
    plt.title('delta z')


def plot_diff_rel(x,
                  y,
                  tri,
                  vals,
                  vals_s,
                  pts_2,
                  quad,
                  v_min=-10,
                  v_max=10,
                  at_point=False):
    """
    This function plots the relative difference between data of a given sample and the mean sample.
    """
    import matplotlib.pyplot as plt

    fig = plt.figure(figsize=(15, 4))

    diff_G = vals_s['G'] - vals['G']
    diff_G_rel = old_div(100 * diff_G, vals['G'])

    plt.subplot(121)
    if not at_point:
        tmp = plot_2D_cut_values_at_elem(x,
                                         y,
                                         tri,
                                         diff_G_rel,
                                         v_min=v_min,
                                         v_max=v_max)
        plot_2D_mesh(pts_2, quad, plt.gca())
    else:
        tmp = plot_2D_cut_values_at_node(x, y, tri, diff_G_rel)
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal')
    plt.clim(v_min, v_max)
    plt.title('delta G relative')

    diff_vM = vals_s['von Mises'] - vals['von Mises']
    diff_vM_rel = old_div(100 * diff_vM, vals['von Mises'])

    plt.subplot(122)
    if not at_point:
        tmp = plot_2D_cut_values_at_elem(x,
                                         y,
                                         tri,
                                         diff_vM_rel,
                                         v_min=v_min,
                                         v_max=v_max)
        plot_2D_mesh(pts_2, quad, plt.gca())
    else:
        tmp = plot_2D_cut_values_at_node(x, y, tri, diff_vM_rel)
    fig.colorbar(tmp, ax=plt.gca(), orientation='horizontal')
    plt.clim(v_min, v_max)
    plt.title('delta von Mises relative')


def main():
    import matplotlib.pyplot as plt

    file_dir = os.path.dirname(__file__)

    file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/fused/katropton_CMAME_tests_plate_SPD_plate_MG/case_'

    n_it = 6
    iters = np.arange(1, 60)  # np.linspace(10,200,n_it,dtype=int)

    n_samples = 8
    cut_of_one_case(1, 8, n_samples, iters, file_MC_base_dir)


if __name__ == "__main__":
    main()
