#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
from katropton.readers import *


def read_iter_all_ensemble_sizes(n_samples,
                                 ensemble_sizes,
                                 file_MC_base_dir,
                                 reduction=False):
    iter = np.zeros((len(ensemble_sizes), n_samples))
    for i in range(0, len(ensemble_sizes)):
        file_MC_base_dir_2 = file_MC_base_dir + '/' + str(
            ensemble_sizes[i]) + '/'
        for j in range(0, int(old_div(n_samples, ensemble_sizes[i]))):
            input_MC_file_dir = file_MC_base_dir_2 + str(j) + '/'
            es = ensemble_sizes[i]
            if reduction:
                es = 1
            iter_indices, residuals = read_Belos(
                input_MC_file_dir + 'belos_out.txt', es)
            for k in range(0, ensemble_sizes[i]):
                iter[i][j * ensemble_sizes[i] + k] = iter_indices[-1]
    return iter


def plot_iter_all_ensemble_sizes(n_samples, iter, print_all=False):
    import matplotlib.pyplot as plt
    plt.figure()
    plt.plot(list(range(0, n_samples)),
             iter[0][:], 'bo', markersize=8, zorder=1)
    ensemble_size = 8
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        plt.plot(list(range(ensemble_size * i, ensemble_size * (i + 1))),
                 iter[1][ensemble_size * i:ensemble_size * (i + 1)],
                 '-ro',
                 linewidth=3,
                 markersize=6,
                 zorder=2)
    if print_all:
        ensemble_size = 16
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            plt.plot(list(range(ensemble_size * i, ensemble_size * (i + 1))),
                     iter[2][ensemble_size * i:ensemble_size * (i + 1)],
                     '-go',
                     linewidth=2,
                     markersize=4,
                     zorder=2)
        ensemble_size = 32
        for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
            plt.plot(list(range(ensemble_size * i, ensemble_size * (i + 1))),
                     iter[3][ensemble_size * i:ensemble_size * (i + 1)],
                     '-ko',
                     linewidth=1,
                     markersize=2,
                     zorder=3)
    plt.xlabel('sample')
    plt.ylabel('Number of iterations to converge')


def write_tex(n_samples, iter, filename):
    f = open(filename, 'w')
    print('\\addplot [c1, mark=*, mark size=2pt, mark options={solid}, only marks]', file=f)
    print('table [row sep=crcr] {%', file=f)
    for i in range(0, n_samples):
        print(str(i) + ' ' + str(iter[0][i]) + '\\\\', file=f)
    print('};', file=f)
    print('\\label{pgfplots:noEP}', file=f)
    ensemble_size = 8
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        print('\\addplot [line width=2pt, c2]', file=f)
        print('table [row sep=crcr] {%', file=f)
        print(str(ensemble_size * i) + ' ' + str(
            iter[1][ensemble_size * i]) + '\\\\', file=f)
        print(str(ensemble_size *
                  (i + 1)) + ' ' + str(iter[1][ensemble_size *
                                               (i + 1) - 1]) + '\\\\', file=f)
        print('};', file=f)
    print('\\label{pgfplots:s8}', file=f)
    ensemble_size = 16
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        print('\\addplot [line width=1.5pt, c3]', file=f)
        print('table [row sep=crcr] {%', file=f)
        print(str(ensemble_size * i) + ' ' + str(
            iter[2][ensemble_size * i]) + '\\\\', file=f)
        print(str(ensemble_size *
                  (i + 1)) + ' ' + str(iter[2][ensemble_size *
                                               (i + 1) - 1]) + '\\\\', file=f)
        print('};', file=f)
    print('\\label{pgfplots:s16}', file=f)
    ensemble_size = 32
    for i in range(0, int(np.ceil(n_samples * 1. / ensemble_size))):
        print('\\addplot [line width=1.pt, c4]', file=f)
        print('table [row sep=crcr] {%', file=f)
        print(str(ensemble_size * i) + ' ' + str(
            iter[3][ensemble_size * i]) + '\\\\', file=f)
        print(str(ensemble_size *
                  (i + 1)) + ' ' + str(iter[3][ensemble_size *
                                               (i + 1) - 1]) + '\\\\', file=f)
        print('};', file=f)
    print('\\label{pgfplots:s32}', file=f)
    f.closed


def main():
    import matplotlib.pyplot as plt

    n_samples = 64
    ensemble_sizes = [1, 8, 16, 32]

    for case_id in [1, 2, 3, 4, 5, 6]:
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Final_MT/katropton_CMAME_tests_plate_MC_MG_all_cases_2_ER_2/case_' + str(
            case_id)
        try:
            iter = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                                file_MC_base_dir, True)
            plot_iter_all_ensemble_sizes(n_samples, iter, True)
            write_tex(n_samples, iter,
                      'MC_MG_case_' + str(case_id) + '_ER.tex')
            plt.grid()
            plt.ylim([100, 400])
            plt.savefig('MC_MG_case_' + str(case_id) + '_ER.png', dpi=400)
            np.savetxt('MC_MG_case_' + str(case_id) + '_ER.txt',
                       np.transpose(iter),
                       delimiter=' ')
            print('MC_MG_case_' + str(case_id) + '_ER')
            iter2 = np.transpose(iter)
            sum_iter = np.zeros((4, 1))
            for i in range(0, 4):
                sum_iter[i] = np.sum(iter2[:, i])
            print(old_div(sum_iter, sum_iter[0]))
        except:
            print(file_MC_base_dir + ' has not all the files')

        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Final_MT/katropton_CMAME_tests_plate_MC_MG_all_cases_2_NER_2/case_' + str(
            case_id)
        try:
            iter = read_iter_all_ensemble_sizes(n_samples, ensemble_sizes,
                                                file_MC_base_dir, False)
            plot_iter_all_ensemble_sizes(n_samples, iter, True)
            write_tex(n_samples, iter,
                      'MC_MG_case_' + str(case_id) + '_NER.tex')
            plt.grid()
            plt.ylim([100, 400])
            plt.savefig('MC_MG_case_' + str(case_id) + '_NER.png', dpi=400)
            np.savetxt('MC_MG_case_' + str(case_id) + '_NER.txt',
                       np.transpose(iter),
                       delimiter=' ')
            iter2 = np.transpose(iter)
            sum_iter = np.zeros((4, 1))
            for i in range(0, 4):
                sum_iter[i] = np.sum(iter2[:, i])
            print(old_div(sum_iter, sum_iter[0]))
        except:
            print(file_MC_base_dir + ' has not all the files')


if __name__ == "__main__":
    main()
