#include "NodesList.h"

#include "wProblem.h"
#include "wMshData.h"
#include "wNode.h"

using namespace katropton;

/**
 * @brief NodesList constructor
 * 
 * This constructor allocates 1 Kokkos::View which stores all the nodes positions.
 * 
 * Arguments:
 * <ul>
 *  <li> pbl: a Problem object,
 *  <li> map: an RCP to a Map object.
 * </ul>
 */
NodesList::NodesList(Problem &pbl, Teuchos::RCP<Map> map)
{
    nodes_number = map->mapNodesWO->getNodeNumElements();

    nodes = Kokkos::View<double **, Kokkos::LayoutRight>("R", nodes_number, 3);
    for (auto i = 0; i < nodes_number; ++i)
        for (auto j = 0; j < 3; ++j)
            nodes(i, j) = pbl.msh->nodes[map->mapNodesWO->getGlobalElement(i)]->pos(j);
}
