#ifndef KATROPTON_PROBLEM_H
#define KATROPTON_PROBLEM_H

#include "katropton.h"
#include "wObject.h"
#include <iostream>
#include <vector>
#include <string>
#include <memory>

#include <mpi.h>

namespace katropton
{

/**
 * @brief Class which is used to specify in Python the thermomechanical to solve.
 */
class KATROPTON_API Problem : public fwk::wSharedObject
{
public:
    std::shared_ptr<tbox::MshData> msh; ///< Mesh structure

    MPI_Comm comm;
#ifndef SWIG
    std::vector<Medium *> media = {};    ///< Materials
    std::vector<Dirichlet *> duBCs = {}; ///< mechanical Dirichlet boundary conditions
    std::vector<Neumann *> nBCs = {};
    std::vector<Contact *> Contacts = {};
    std::vector<Source *> Sources = {};
    std::vector<std::shared_ptr<RandomVariable>> RandomVariables = {};
    std::vector<Weight *> Weights = {};
#endif
    Problem(std::shared_ptr<tbox::MshData> _msh, MPI_Comm _comm);
    ~Problem() { std::cout << "~Problem()\n"; }

    void add(std::shared_ptr<RandomVariable> r);
#ifndef SWIG
    virtual void write(std::ostream &out) const;
#endif
};

} // namespace katropton

#endif //KATROPTON_PROBLEM_H
