#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
import katropton as m
import tbox
import tbox.gmsh as gmsh
#import matplotlib.pyplot as plt

import katropton.utilities as u


def main():
    comm, rank, size = u.import_MPI()

    from fwk.wutils import parseargs
    args = parseargs()

    from PyTrilinos import Teuchos
    import os
    import numpy as np

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    msh = u.fuse_meshes('L_shape_1.geo', 'L_shape_1', 'L_shape_2.geo',
                        'L_shape_2', 'L_shape_fused', file_dir, work_dir, comm, rank, size)

    ensemble_size = 1

    pbl = m.Problem(msh, comm)

    E = 10. * np.ones(ensemble_size)
    nu = 0.4 * np.ones(ensemble_size)
    k = 0.114
    d = 0.
    m.Medium(pbl, "body", "test", E, nu, k, d)

    m.Dirichlet(pbl, "clamping", "Clamped", 1, 0.,
                1, 0., 1, 0., 0, 0., ensemble_size)
    #m.Dirichlet(pbl, "load", "Clamped",1,0.,1,2.,1,0.,0,0.,ensemble_size)

    norm = tbox.Vector3d(0, 0, -1)
    cont = m.Contact(pbl, "outer part boundary 1", "contact", norm)
    cont.setMeshTying()
    cont.setMaster(pbl, "inner edge boundary 1", norm)
    cont1 = m.Contact(pbl, "inner edge boundary 2", "contact", norm)
    cont1.setMeshTying()
    cont1.setMaster(pbl, "outer part boundary 2", norm)
    cont2 = m.Contact(pbl, "outer part boundary 3", "contact", norm)
    cont2.setMeshTying()
    cont2.setMaster(pbl, "inner edge boundary 3", norm)
    cont3 = m.Contact(pbl, "inner edge boundary 4", "contact", norm)
    cont3.setMeshTying()
    cont3.setMaster(pbl, "outer part boundary 4", norm)
    f = -0.002 * np.ones(ensemble_size)
    zero = np.zeros(ensemble_size)

    m.Neumann(pbl, "load", "load", 0, zero, 1,
              f, 0, zero, 0, zero, ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 10
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList["MueLu xml file name"] = file_dir + '/direct_solver.xml'
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 1

    #solverList['Write matrix and vectors'] = True
    solverList['Write txt files'] = True

    nThreads = u.Initialize_Kokkos()

    slv = m.IterativeSolver(pbl, solverList, 3, ensemble_size)

    slv.start()

    u.Finalize_Kokkos()


if __name__ == "__main__":
    main()
