#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import katropton as m
import tbox
import tbox.gmsh as gmsh
import katropton.utilities as u
#import matplotlib2tikz
# import katropton.viewer as katv # Error occurs after that


def analytical(E, nu, L, W, H, d, p, x):

    EI = old_div((E * W * H**3), 12.)
    q = p * W
    q_1 = 24*EI*d/L**4
    q_2 = 72*EI*d/L**4
    a = (72*EI*d/q)**(0.25)

    print(a)

    print(E)
    print(nu)
    print(L)
    print(W)
    print(H)
    print(d)
    print(p)

    if q < q_1:
        case_id = 1
        sol = -(old_div(q, (24.*EI)))*(x**4-4.*L*x**3+4.*L**2*x**2)
    if q >= q_1:
        case_id = 2
        sol = -(old_div(q, (24.*EI)))*(x**4-2.*L*x**3+L**2*x**2) + \
            2.*d*x**3/(L**3)-3.*d*x**2/(L**2)
    if q >= q_2:
        case_id = 3
        sol = -(old_div(q, (24.*EI))) * \
            (x**4-(old_div(8., 3.))*a*x**3+2.*a**2*x**2)
        for i in range(0, len(x)):
            if x[i] >= a:
                sol[i] = -d

    print("q")
    print(q)
    print(q_1)
    print(q_2)

    return case_id, sol


def analytical_ensemble(E, nu, L, W, H, d, p, x, ensemble_size):

    import numpy as np

    case_id = np.empty([ensemble_size, 1])
    sol = np.empty([len(x), ensemble_size])

    for i in range(0, ensemble_size):
        case_id[i], sol[:, i] = analytical(E[i], nu[i], L, W, H, d, p[i], x)

    return case_id, sol


def main():
    comm, rank, size = u.import_MPI()

    from fwk.wutils import parseargs
    args = parseargs()

    from PyTrilinos import Teuchos
    import numpy as np
    #import matplotlib.pyplot as plt

    DOFperNode = 3

    display = True

    # Mesh

    geo_name = 'beam.geo'
    mesh_name = 'beam.msh'
    partitioned_mesh_name = 'beam_k.msh'

    par = {}
    par['L'] = 50.
    par['W'] = 5.
    par['H'] = 5.
    par['d'] = 1.
    par['n_L'] = 20
    par['n_hW'] = 2
    par['n_hH'] = 2

    ensemble_size = 8

    import shlex
    import subprocess
    import os

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    pars = {'L': par['L'], 'W': par['W'], 'H': par['H'], 'd': par['d'],
            'n_L': par['n_L'], 'n_hW': par['n_hW'], 'n_hH': par['n_hH']}

    msh, mesh_name, partitioned_mesh_name = u.read_mesh_MPI(
        geo_name, file_dir, work_dir, comm, rank, size, pars)

    for n1 in msh.nodes:
        n1.row = n1.no-1

    # Problem

    #msh = gmsh.MeshLoader(mesh_name,work_dir).execute()

    pbl = m.Problem(msh, comm)

    E = 21000 * np.ones(ensemble_size)
    nu = 0.29 * np.ones(ensemble_size)
    k = 0
    d = 0

    # for i in range(0,ensemble_size):
    #    E[i] = 21000+i*5000

    m.Medium(pbl, "Beam", "Mat 1", E, nu, k, d)
    m.Medium(pbl, "Rigid body", "Mat 1", E, nu, k, d)

    dx = np.zeros(ensemble_size)
    dy = np.zeros(ensemble_size)
    dy2 = np.linspace(0, -2., ensemble_size)
    dz = 0*np.linspace(0, -2., ensemble_size)
    dz2 = np.zeros(ensemble_size)
    dT = np.zeros(ensemble_size)
    m.Dirichlet(pbl, "Rigid clamped surface",
                "Clamped 1", 1, dx, 1, dy, 1, dz, 0, dT)
    m.Dirichlet(pbl, "Clamped end", "Clamped 1", 1, dx, 1, dy, 1, dz2, 0, dT)
    m.Dirichlet(pbl, "Free end", "Clamped 1", 1, 0.,
                0, 0., 0, 0., 0, 0., ensemble_size)

    #p = 0.82* np.ones(ensemble_size)

    #p = np.linspace(0.1, 0.835, num=ensemble_size)
    # np.linspace(1.2, 0., num=ensemble_size) # 3.6 * np.ones(ensemble_size) #np.linspace(1.5, 2., num=ensemble_size)
    p = 0.9 * np.ones(ensemble_size)
    for i in range(0, ensemble_size):
        p[i] = 1.7+(7-i)*0.025

    p[0] = 2.5

    zero = np.zeros(ensemble_size)

    m.Neumann(pbl, "Loaded surface", "Load 1", 0, zero,
              0, zero, 1, -p, 0, zero, ensemble_size)

    norm1 = tbox.Vector3d(0, 0, -1)
    norm2 = tbox.Vector3d(0, 0, 1)
    dg = 0*np.linspace(0., 1., num=ensemble_size)

    dg[0] = 0.

    a = 49.5
    initiallyClosedNodesList = np.array([], dtype=np.float)
    #tag_name = "Center of cross section"
    for el in msh.ntags["Beam contact zone"].elems:
        for n1 in el.nodes:
            node_id = n1.no-1
            if msh.nodes[node_id].pos[0] >= a:
                initiallyClosedNodesList = np.append(
                    initiallyClosedNodesList, node_id)

    cont1 = m.Contact(pbl, "Beam contact zone", "contact", norm1, dg)
    #cont1.setSlaveBoundary(work_dir+"/"+partitioned_mesh_name, "Beam contact boundary")
    #cont1.setMasterBoundary(work_dir+"/"+partitioned_mesh_name, "Rigid surface boundary")
    cont1.setInitialyOpen()
    cont1.setInitiallyClosedNodes(initiallyClosedNodesList)
    cont1.setDualShapeFunctions()
    cont1.setSlaveDirichlet(
        work_dir+"/"+partitioned_mesh_name, "Beam contact Dirichlet")
    cont1.setSignoriniContact()
    cont1.setMaster(pbl, "Rigid surface", norm2)

    # Solver

    solverList = Teuchos.ParameterList()
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 100000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList["MueLu xml file name"] = file_dir + '/multigrid_2_lvls_3_1.xml'
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 4
    solverList['Write txt files'] = True
    slv = m.IterativeSolver(pbl, solverList,
                            DOFperNode, ensemble_size)

    nThreads = u.Initialize_Kokkos()

    slv.start()

    active_set_iteration = slv.active_set_iteration

    u.Finalize_Kokkos()

    if rank == 0:
        import fwk.testing as tst
        tests = tst.CTests()
        tests.add(tst.CTest('Number of active set iterations',
                            active_set_iteration, 3))
        tests.run()


if __name__ == "__main__":
    main()
