#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import division

from builtins import str
from builtins import range
from past.utils import old_div
from katropton.CMAME import model

import katropton as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katropton.utilities as u

from katropton.readers import read_Belos
from fwk.testing import *


def main():
    """
    This loads the mesh and define the case ids and ensemble sizes
    """
    comm, rank, size = m.utilities.import_MPI()

    if size == 1:
        args = parseargs()

        file_dir = os.path.dirname(__file__)
        work_dir = os.getcwd()

        # Units in the meshes are in mm
        msh = u.fuse_meshes('plate_1.geo', 'plate_1_tmp', 'plate_2.geo',
                            'plate_2_tmp', 'plate_fused',
                            file_dir + '/../CMAME/mesh', work_dir, comm, rank,
                            size)

        np.random.seed(42)
        N = 1
        num_random_variables = 20
        randomVariable = np.zeros((N, num_random_variables))

        p = model.get_parameters(0, 0)
        p['Write txt files'] = True
        p['MueLu xml template file name'] = '/../preconditioners/SIMPLE_gs_direct_lvl.xml.in'

        ensemble_size = 1

        pm = model.plate_model(ensemble_size, randomVariable, msh, comm,
                               file_dir, work_dir, p)

        nThreads = u.Initialize_Kokkos()
        pm.run()
        u.Finalize_Kokkos()

        iter_indices, residuals = read_Belos(work_dir + '/belos_out.txt', 1)

        tests = CTests()
        nIterations = iter_indices[-1]
        tests.add(CTest('Has converged', nIterations, 97, 1e-2))
        tests.run()


if __name__ == "__main__":
    main()