#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from __future__ import print_function
from builtins import str
from builtins import range
import numpy as np
import os
import shutil

import sys

import matplotlib.pylab as plt

import params.src.indices as indices


def main():
    d = 1

    M = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        M[j] = 10

    #--------------------------------------------------------
    N = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        N[j] = 10

    #--------------------------------------------------------
    from params.src.legendre_norm import phi                      # monomials or legendre
    import params.src.gauss as tp                             # just gauss    
    #========================================================

    X, w = tp.grid(M)  

    m_max = 1
    n_max = 1 
    for j in range(0,d):
        m_max = m_max*M[j] 
        n_max = n_max*N[j]

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    xi = np.zeros((d,))

    m_i = 0

    Z = np.zeros((m_max,n_max))
    W = np.zeros((m_max,m_max))   


    while np.all(i != -i_one):

        for j in range(0,d):
            xi[j] = X[i[j],j]

        Z[m_i,:] = np.transpose(phi(xi,N,n_max,d))

        W[m_i,m_i] = 1
        for j in range(0,d):
            W[m_i,m_i] = W[m_i,m_i]*w[i[j],j]
       
        p,i = indices.my_next(p,i,M,d)
        m_i = m_i + 1     
        text = str(m_i)+' / '+str(m_max)
        sys.stdout.write("\r"+text)
        sys.stdout.flush()

    sys.stdout.write("\n")
    ZWZ = np.dot(np.transpose(Z),np.dot(W,Z))

    tmp = np.abs(ZWZ-np.diag(np.diag(ZWZ)))
    
    print(np.amax(np.abs(tmp)))

    from fwk.wutils import parseargs
    args = parseargs()
    if not args.nogui:
        fig = plt.figure()
        ax = fig.add_subplot(1,1,1)
        ax.set_aspect('equal')
        plt.imshow(tmp, interpolation='nearest', cmap='Greys')
        plt.colorbar()
        plt.show()


if __name__ == "__main__":
    main()
