/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @brief Return the shape function derivatives at Gauss point k
 * 
 * dff[k](i, j) contains the ith derivative of the shape function associated to node j
 */
inline Eigen::MatrixXd const &Cache::getDsf(size_t k)
{
    return dff[k];
}

/**
 * @brief Return the shape functions at Gauss point k
 * 
 * ff[k](i) contains the shape function associated to node i
 */
inline Eigen::VectorXd const &Cache::getSf(size_t k)
{
    return ff[k];
}
