/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WMSHDATA_H
#define WMSHDATA_H

#include "tbox.h"
#include "wObject.h"
#include <string>
#include <vector>
#include <map>

namespace tbox
{

/**
 * @brief Mesh data structure
 * @authors Romain Boman, Adrien Crovato, Kim Liegeois
 */
class TBOX_API MshData : public fwk::wSharedObject
{
    bool verb;

public:
    std::vector<Node *> nodes;    ///< nodes
    std::vector<Element *> elems; ///< elements
    std::string name;             ///< name

    std::map<int, Tag *> ptags;         ///< physical tags
    std::map<int, Tag *> etags;         ///< elementary tags
    std::map<int, Tag *> parts;         ///< partition tags
    std::map<std::string, Tag *> ntags; ///< named tags

    MshData();
    virtual ~MshData();

    void clear();
    void debug(bool v = true) { verb = v; }
    virtual void write(std::ostream &out) const override;
};

} // namespace tbox

#endif //WMSHDATA_H
